#include <math.h>

#include "cap.h"
#include "specialimage.h"
#include "game.h"
#include "collision.h"
#include "soccergame.h"

bool Cap::moving=false;
float Cap::graze=0.0;

void Cap::init()
{
	if(team==SG::RIGHT)
		ag=new SpecialImage(this,"cap_right.png");
	else
		ag=new SpecialImage(this,"cap_left.png");

	ag->init();

	r=ag->get_w()/2;

	ang=0;
	collisionable=true;
	layer=5;
}

void Cap::move()
{
	if(vel.module()!=0.0)
	{
		Cap::moving=true;
		vel=vel+accel;
		vel=multiply(graze*(1-mass),vel);
		if(accel.module()!=0.0)
		{
			accel=multiply(graze,accel);
			float cos_alpha=escalate(normalize(accel),normalize(vel));
			ang+=10*acos(cos_alpha)/M_PI;

			if(ang<-180)
				ang=ang+360;
			else if(ang>180)
				ang=ang-360;

			((SpecialImage*)ag)->rotate(ang);
		}
		if(vel.module()<0.5)
		{
			setVel(zero);
			setAccel(zero);
		}
		/*
		Vector aux=normalize(accel);
		accel=multiply(accel.module()-graze,vel);
		if(vel.module()<0.5)
			vel=Vector(zero);
		else 
			vel=add(vel,accel);
			*/
	}
}

void Cap::getCollision(Collision &c)
{
	//Si chocamos contra otra chapa o la pelota
	if(c.type==ROUND)
	{
		//Direccion de la bola que esta parada es el vector
		//diferencia entre los vectores posicion parado y
		//moviendose
		Vector par=sub(c.b->getPos(),c.a->getPos());

		//Direccion de la bola que se esta moviendo es el
		//inverso del vector reflejo respecto a la velparado
		Vector mov=reflect(c.a->getVel(),par);
		mov=reverse(mov);

		//Normalizamos los vectores
		par=normalize(par);
		mov=normalize(mov);

		//Ahora lo multiplicamos por la cantidad de moviento
		//y una constante que en cada es cos alpha o sen alpha
		float cos_alpha=escalate(mov,par);
		//Nos interesa en valor absoluto
		if(cos_alpha<0)
			cos_alpha=(-cos_alpha);

		//Comprobamos si es la que golpea
		if(c.a==this)
		{
			float module=(1-cos_alpha)*vel.module()*c.b->getMass()/mass;
			vel=multiply(mov,module);

			//Le quitamos el efecto
			setAccel(zero);
		}
		//Es la golpeada
		else
		{
			float module=cos_alpha*c.a->getVel().module()*c.a->getMass()/
				c.b->getMass();
			vel=multiply(par,module);

			//Le aplicamos el efecto
			accel=c.b->getAccel();
		}
	}
	//Chocamos contra la pared
	else if(c.type==WALL)
	{
		BoxActor *line=(BoxActor*)c.b;

		Vector n=line->getSize();
		n=normalize(n);
		n=rotateRight(n);
		
		//El nuevo vector de velocidad del circulo es el inverso del reflejo 
		//del actual respecto a la normal de linea
		vel=reflect(vel,n);
		vel=reverse(vel);

		//Le quitamos el efecto
		setAccel(zero);
	}
	//Chocamos contra un poster
	else
	{
		BoxActor *line=(BoxActor*)c.b;

		Vector dif1=getPos()-line->getMin();
		Vector dif2=getPos()-line->getMax();

		//Ponemos en dif1 el mas cercano al poster
		if(dif1.module()>dif2.module())
			dif1=dif2;
		
		dif1=normalize(dif1);
		vel=multiply(vel.module(),dif1);

		//Le quitamos el efecto
		setAccel(zero);
	}
	
}
