#include <SDL.h>
#include <SDL_rotozoom.h>

#include <string>

using namespace std;

#include "image.h"
#include "utils.h"
#include "rendermanager.h"
#include "actor.h"
#include "collision.h"

void Image::init()
{
	img=LoadImgAlpha(name);
	center=owner->getCenter();
}

void Image::activate()
{
	rect=owner->getRect();
	center=owner->getCenter();
	g->rendermanager->addToDraw(rect);
}

void Image::update()
{
	if(center!=owner->getCenter())
	{
		//Poner el fondo donde estaba antes
		g->rendermanager->addToDraw(rect);

		center=owner->getCenter();
		rect=owner->getRect();

		//Poner el fondo donde va a estar ahora
		g->rendermanager->addToDraw(rect);
	}
}

void Image::desactivate()
{
	g->rendermanager->addToDraw(rect);
}

void Image::flip()
{
	SDL_Surface *aux=img;
	img=rotozoomSurface(img,180,1,SMOOTHING_ON);
	SDL_FreeSurface(aux);
	g->rendermanager->addToDraw(rect);
	change=true;
}

void Image::draw()
{
	SDL_Rect r=rect;
	SDL_BlitSurface(img,NULL,SDL_GetVideoSurface(),&r);
}

void Image::draw(SDL_Rect &r)
{
	SDL_Rect a=rect;

	if(isCollision(owner,r)!=NO_COLLISION)
		puts("collision");
	else
		puts("ennnnnnnn");

	SDL_Rect src;
	SDL_Rect dst;

	//Rellenamos los rectangulos src y dst
	//para pintar solo lo que se ha modificado
	//Si el actor esta a la derecha del rect
	if(r.x<=a.x)
	{
		dst.x=a.x;
		src.x=0;

		//Si se sale por la derecha pintamos solo la parte
		//que queda dentro del rect
		if(r.x+r.w<a.x+a.w)
		{
			puts("1 / 1");
			dst.w=r.x+r.w-a.x;
			src.w=r.x+r.w-a.x;
		}
		//Sino pintamos todo el ancho de la figura
		else
		{
			puts("1 / 2");
			dst.w=a.w;
			src.w=a.w;
		}
	}
	//Si el actor esta a la izquierda del rect
	else
	{
		dst.x=r.x;
		//Si el actor esta fuera de la pantalla
		if(a.x<0)
			src.x=r.x-a.x;
		else
			src.x=r.x+a.x;

		//Si se sale por la derecha pintamos solo la parte
		//que queda dentro del rect
		if(r.x+r.w<=a.x+a.w)
		{
			puts("2 / 1");
			dst.w=r.w;
			src.w=r.w;
		}
		//Sino pintamos todo el ancho de la figura
		else
		{
			puts("2 / 2");
			printf("actor %d %d %u %u\n",a.x,a.y,a.w,a.h);
			printf("update %d %d %u %u\n",r.x,r.y,r.w,r.h);
			dst.w=a.x+a.w-dst.x;
			src.w=a.x+a.w-dst.x;
		}
	}

	//Si el actor esta por debajo del rectangulo
	if(r.y<=a.y)
	{
		dst.y=a.y;
		src.y=0;

		//Si el actor se sale por debajo del rect pinto solo
		//la parte que queda dentro
		if(r.y+r.h<a.y+a.h)
		{
			dst.h=r.y+r.h-a.y;
			src.h=r.y+r.h-a.y;
		}
		//Si el actor se queda dentro pintamos todo el alto de
		//la imagen
		else
		{
			dst.h=a.h;
			src.h=a.h;
		}
	}
	//Si el actor esta por encima del rectangulo
	else
	{
		dst.y=r.y;
		if(a.y<0)
			src.y=r.y+a.y;
		else
			src.y=r.y-a.y;

		//Si el actor se sale por debajo del rect pinto solo
		//la parte que queda dentro
		if(r.y+r.h<=a.y+a.h)
		{
			dst.h=r.h;
			src.h=r.h;
		}
		//Si el actor se queda dentro pintamos todo el alto de
		//la imagen
		else
		{
			dst.h=a.y+a.h-dst.y;
			src.h=a.y+a.h-dst.y;
		}
	}

	printf("%d %d %u %u\n",dst.x,dst.y,dst.w,dst.h);
	puts("");
	SDL_BlitSurface(img,&src,SDL_GetVideoSurface(),&dst);
	//draw();
}
